import mysql.connector
import config


class DBHelper:
    def __init__(self):
        """Connect to MySQL Database"""
        try:
            self.conn = mysql.connector.connect(
                host=config.DB_HOST,
                user=config.DB_USER,
                password=config.DB_PASSWORD,
                database=config.DB_NAME
            )
            self.cursor = self.conn.cursor(dictionary=True)
            print("✅ Successfully connected to the MySQL database.")
            self.setup()
        except mysql.connector.Error as err:
            print(f"❌ Error connecting to MySQL: {err}")

    # ✅ Setup Database Schema
    def setup(self):
        """Create required tables if they don't exist"""
        try:
            # Table for Users
            self.cursor.execute('''
                CREATE TABLE IF NOT EXISTS users (
                    user_id BIGINT PRIMARY KEY,
                    max_file_size_mb FLOAT DEFAULT NULL,
                    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
                )
            ''')

            # Table for Admin States
            self.cursor.execute('''
                CREATE TABLE IF NOT EXISTS admin_states (
                    user_id BIGINT PRIMARY KEY,
                    state VARCHAR(255) NOT NULL,
                    updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
                )
            ''')

            self.conn.commit()
        except mysql.connector.Error as err:
            print(f"❌ Error creating tables: {err}")

    # ✅ Ensure User Exists
    def user_exists(self, user_id: int) -> bool:
        """Check if a user exists in the database"""
        try:
            self.cursor.execute('SELECT * FROM users WHERE user_id = %s', (user_id,))
            return self.cursor.fetchone() is not None
        except mysql.connector.Error as err:
            print(f"❌ Error checking user existence: {err}")
            return False

    # ✅ Add New User
    def add_user(self, user_id: int):
        """Add a new user to the database"""
        try:
            if not self.user_exists(user_id):
                self.cursor.execute('INSERT INTO users (user_id) VALUES (%s)', (user_id,))
                self.conn.commit()
        except mysql.connector.Error as err:
            print(f"❌ Error adding user: {err}")

    # ✅ Set Max File Size for a User
    def set_user_max_file_size(self, user_id: int, max_size: float):
        """Set maximum file size for a user"""
        try:
            self.add_user(user_id)  # Ensure user exists
            self.cursor.execute('UPDATE users SET max_file_size_mb = %s WHERE user_id = %s', (max_size, user_id))
            self.conn.commit()
            print(f"✅ Max file size set to {max_size} MB for user {user_id}")
        except mysql.connector.Error as err:
            print(f"❌ Error setting max file size: {err}")

    def get_user_max_file_size(self, user_id: int) -> float:
        """Get maximum file size for a user"""
        try:
            self.cursor.execute('SELECT max_file_size_mb FROM users WHERE user_id = %s', (user_id,))
            result = self.cursor.fetchone()
            if result and result['max_file_size_mb'] is not None:
                return result['max_file_size_mb']
            return config.MAX_FILE_SIZE_MB  # Default from config
        except mysql.connector.Error as err:
            print(f"❌ Error fetching max file size: {err}")
            return config.MAX_FILE_SIZE_MB


    # ✅ Get Total User Count
    def get_user_count(self) -> int:
        """Get the total number of registered users"""
        try:
            self.cursor.execute('SELECT COUNT(*) as count FROM users')
            result = self.cursor.fetchone()
            return result['count'] if result else 0
        except mysql.connector.Error as err:
            print(f"❌ Error fetching user count: {err}")
            return 0

    # ✅ Set Admin State
    def set_admin_state(self, user_id: int, state: str):
        """Set admin state"""
        try:
            self.cursor.execute('''
                INSERT INTO admin_states (user_id, state)
                VALUES (%s, %s)
                ON DUPLICATE KEY UPDATE state = %s, updated_at = CURRENT_TIMESTAMP
            ''', (user_id, state, state))
            self.conn.commit()
        except mysql.connector.Error as err:
            print(f"❌ Error setting admin state: {err}")

    # ✅ Get Admin State
    def get_admin_state(self, user_id: int) -> str:
        """Get admin state"""
        try:
            self.cursor.execute('SELECT state FROM admin_states WHERE user_id = %s', (user_id,))
            result = self.cursor.fetchone()
            return result['state'] if result else None
        except mysql.connector.Error as err:
            print(f"❌ Error fetching admin state: {err}")
            return None

    # ✅ Clear Admin State
    def clear_admin_state(self, user_id: int):
        """Clear admin state"""
        try:
            self.cursor.execute('DELETE FROM admin_states WHERE user_id = %s', (user_id,))
            self.conn.commit()
        except mysql.connector.Error as err:
            print(f"❌ Error clearing admin state: {err}")

    # ✅ Close Database Connection
    def close(self):
        """Close the database connection"""
        if self.conn.is_connected():
            self.cursor.close()
            self.conn.close()
            print("✅ Database connection closed.")
